package com.sd.business.service.bom.impl;

import cn.hutool.core.util.ReUtil;
import cn.hutool.core.util.StrUtil;
import com.ruoyi.common.constant.StatusConstant;
import com.ruoyi.common.core.domain.BaseIdPo;
import com.sd.business.entity.bom.dto.BomSpecPriceQueryDto;
import com.sd.business.entity.bom.enums.BomSpecPriceConfigCodeEnum;
import com.sd.business.entity.bom.po.BomSpecPriceConfig;
import com.sd.business.entity.bom.vo.BomSpecPriceDetailsVo;
import com.sd.business.mapper.bom.BomSpecPriceConfigMapper;
import com.sd.business.service.bom.BomSpecPriceConfigService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.stereotype.Service;
import com.sd.business.entity.bom.vo.BomSpecPriceConfigVo;
import com.sd.business.entity.bom.dto.BomSpecPriceConfigSelectDto;
import com.ruoyi.common.utils.wrapper.IWrapper;
import com.sd.business.entity.bom.dto.BomSpecPriceConfigDto;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * <p>
 * bom规格价格配置 服务实现类
 * </p>
 *
 * @author 
 * @since 2023-12-25
 */
@Service
public class BomSpecPriceConfigServiceImpl extends ServiceImpl<BomSpecPriceConfigMapper, BomSpecPriceConfig> implements BomSpecPriceConfigService {

    @Override
    public List<BomSpecPriceConfigVo> getList(BomSpecPriceConfigSelectDto dto) {
        IWrapper<BomSpecPriceConfig> wrapper = getWrapper();
        wrapper.orderByDesc("bspc", BomSpecPriceConfig::getId);
        wrapper.eq("bspc", BomSpecPriceConfig::getCode, dto.getCode());
        List<BomSpecPriceConfigVo> list = this.baseMapper.getList(wrapper);
        return list;
    }

    @Override
    public void add(BomSpecPriceConfigDto bomSpecPriceConfigDto) {
        this.save(bomSpecPriceConfigDto);
    }

    @Override
    public void edit(BomSpecPriceConfigDto bomSpecPriceConfigDto) {
        this.updateById(bomSpecPriceConfigDto);
    }

    @Override
    public void delete(Long id) {
        this.removeById(id);
    }

    @Override
    public BomSpecPriceDetailsVo getBomSpecPrice(BomSpecPriceQueryDto dto) {
        // 计算公式=（基材价格+品类+特性+颜色价格）*[（长+裁切配置）*（宽+裁切配置）/10000]*（厚度+压纹配置）*（1+损耗）+加工费+运费
        BomSpecPriceDetailsVo vo = new BomSpecPriceDetailsVo();
        vo.setLength(dto.getLength());
        vo.setWidth(dto.getWidth());
        vo.setHeight(dto.getHeight());
        List<BomSpecPriceConfig> list = this.list();
        Map<Long, BomSpecPriceConfig> configDetailMap = list.stream().collect(Collectors.toMap(BaseIdPo::getId, Function.identity()));
        Map<String, List<BomSpecPriceConfig>> configCodeMap = list.stream().collect(Collectors.groupingBy(BomSpecPriceConfig::getCode));

        // 等级
        BomSpecPriceConfig levelDetail = configDetailMap.get(dto.getLevelConfigId());
        vo.setBaseMaterialPrice(levelDetail.getPrice());

        // 品类
        BomSpecPriceConfig categoryDetail = configDetailMap.get(dto.getCategoryConfigId());
        vo.setCategoryPrice(categoryDetail.getPrice());

        // 特性
        BigDecimal characterPrice = BigDecimal.ZERO;
        if (StrUtil.isNotBlank(dto.getCharacterConfigIds())) {
            String[] characterConfigIdArr = dto.getCharacterConfigIds().split(",");
            for (String configId : characterConfigIdArr) {
                BomSpecPriceConfig embossingDetail = configDetailMap.get(Long.parseLong(configId));
                characterPrice = characterPrice.add(embossingDetail.getPrice());
            }
        }
        vo.setCharacterPrice(characterPrice);

        // 颜色
        BigDecimal colourPrice = BigDecimal.ZERO;
        String colourConfigIds = dto.getColourConfigIds();
        List<String> colourConfigIdList = Arrays.asList(colourConfigIds.split(","));
        for (String configId : colourConfigIdList) {
            BomSpecPriceConfig colourDetail = configDetailMap.get(Long.parseLong(configId));
            if (colourConfigIdList.size() > 1) {
                colourPrice = colourPrice.add(colourDetail.getPrice().divide(BigDecimal.valueOf(2), 2, RoundingMode.HALF_UP));
            } else {
                colourPrice = colourPrice.add(colourDetail.getPrice());
            }
        }
        vo.setColourPrice(colourPrice);

        // 获取裁切配置
        BigDecimal cropLength = BigDecimal.ZERO;
        BigDecimal cropWidth = BigDecimal.ZERO;
        List<BomSpecPriceConfig> bomSpecCropConfigList = configCodeMap.getOrDefault(BomSpecPriceConfigCodeEnum.CROP_CONFIG.getKey(), Collections.emptyList());
        BomSpecPriceConfig cropConfig = bomSpecCropConfigList.stream()
                .filter(item -> item.getName().contains(categoryDetail.getName()))
                .findAny().orElse(null);
        if (cropConfig != null) {
            cropLength = cropConfig.getLength();
            cropWidth = cropConfig.getWidth();
        }
        vo.setCropLength(cropLength);
        vo.setCropWidth(cropWidth);

        // 获取压纹配置
        BigDecimal embossingHeight = BigDecimal.ZERO;
        BomSpecPriceConfig embossingConfig = configDetailMap.get(dto.getEmbossing());
        if (embossingConfig != null) {
            embossingHeight = embossingConfig.getHeight();
        }
        vo.setEmbossingHeight(embossingHeight);

        // 获取损耗配置
        BigDecimal depletion = BigDecimal.ZERO;
        String depletionCode;
        if (Objects.equals(dto.getBomClassify(), StatusConstant.YES)) {
            depletionCode = BomSpecPriceConfigCodeEnum.YOGA_MAT_DEPLETION_CONFIG.getKey();
        } else {
            depletionCode = BomSpecPriceConfigCodeEnum.SKIP_MAT_DEPLETION_CONFIG.getKey();
        }
        List<BomSpecPriceConfig> bomSpecDepletionConfigList = configCodeMap.getOrDefault(depletionCode, Collections.emptyList());
        BomSpecPriceConfig depletionConfig = bomSpecDepletionConfigList.stream()
                .filter(item -> item.getName().contains(categoryDetail.getName()))
                .findAny().orElse(null);
        if (depletionConfig != null) {
            depletion = depletionConfig.getDepletion();
        }
        // 获取附加配置
        List<BomSpecPriceConfig> additionalConfigList = bomSpecDepletionConfigList.stream()
                .filter(item -> Objects.equals(item.getType(), StatusConstant.YES)).collect(Collectors.toList());

        // 瑜伽垫对比宽度，跳绳垫对比级别
        BomSpecPriceConfig additionalConfig;
        if (Objects.equals(dto.getBomClassify(), StatusConstant.YES)) {
            additionalConfig = additionalConfigList.stream()
                    .filter(item -> this.operatorCompareTo(item.getName(), dto.getWidth()))
                    .findAny().orElse(null);
        } else {
            additionalConfig = additionalConfigList.stream()
                    .filter(item -> item.getName().contains(levelDetail.getName()))
                    .findAny().orElse(null);
        }

        // 附加配置匹配成功，增加损耗
        if (additionalConfig != null) {
            depletion = depletion.add(additionalConfig.getDepletion());
        }
        vo.setDepletion(depletion);

        // 获取加工费配置
        List<BomSpecPriceConfig> bomSpecProcessingFeeConfigList = configCodeMap.getOrDefault(BomSpecPriceConfigCodeEnum.PROCESSING_FEE_CONFIG.getKey(), Collections.emptyList());
        BigDecimal processingFee = bomSpecProcessingFeeConfigList.stream()
                .map(BomSpecPriceConfig::getPrice)
                .reduce(BigDecimal.ZERO, BigDecimal::add);
        // 如果是热复合时，增加复合费用
        if (Objects.equals(categoryDetail.getName(), "热复合")) {
            List<BomSpecPriceConfig> compoundingCostConfigList = configCodeMap.getOrDefault(BomSpecPriceConfigCodeEnum.COMPOUNDING_COST_CONFIG.getKey(), Collections.emptyList());
            BomSpecPriceConfig priceConfig = compoundingCostConfigList.stream()
                    .filter(item -> this.operatorCompareTo(item.getName(), dto.getWidth()))
                    .findAny().orElse(null);
            if (priceConfig != null) {
                processingFee = processingFee.add(priceConfig.getPrice());
            }
        }

        vo.setProcessingFee(processingFee);

        // 获取运费配置
        List<BomSpecPriceConfig> bomSpecFreightConfigList = configCodeMap.getOrDefault(BomSpecPriceConfigCodeEnum.FREIGHT_CONFIG.getKey(), Collections.emptyList());
        BigDecimal freightFee = BigDecimal.ZERO;
        BomSpecPriceConfig freightConfig = bomSpecFreightConfigList.stream()
                .filter(item -> this.operatorCompareTo(item.getName(), dto.getWidth())).findAny().orElse(null);
        if (freightConfig != null) {
            freightFee = freightConfig.getPrice();
        }
        vo.setFreightFee(freightFee);

        BigDecimal price = vo.getBaseMaterialPrice()
                .add(vo.getCategoryPrice())
                .add(vo.getCharacterPrice())
                .add(vo.getColourPrice());

        BigDecimal unitPrice = price
                .multiply(vo.getLength().add(vo.getCropLength())
                        .multiply(vo.getWidth().add(vo.getCropWidth()))
                        .divide(BigDecimal.valueOf(10000), 10, RoundingMode.HALF_UP))
                .multiply(dto.getHeight().add(vo.getEmbossingHeight()))
                .multiply(BigDecimal.ONE.add(vo.getDepletion().multiply(BigDecimal.valueOf(0.01))))
                .add(vo.getProcessingFee())
                .add(vo.getFreightFee())
                .setScale(2, RoundingMode.HALF_UP);
        vo.setUnitPrice(unitPrice);
        return vo;
    }

    /**
     * 对比
     * @param rule 规则
     * @param value
     * @return
     */
    private boolean operatorCompareTo(String rule, BigDecimal value) {
        Integer number = ReUtil.getFirstNumber(rule);
        if (rule.startsWith(">=") || rule.startsWith("≥")) {
            return value.compareTo(BigDecimal.valueOf(number)) >= 0;
        } else if (rule.startsWith(">")) {
            return value.compareTo(BigDecimal.valueOf(number)) > 0;
        } else if (rule.startsWith("<=") || rule.startsWith("≤")) {
            return value.compareTo(BigDecimal.valueOf(number)) <= 0;
        } else if (rule.startsWith("<")) {
            return value.compareTo(BigDecimal.valueOf(number)) < 0;
        } else {
            return value.compareTo(BigDecimal.valueOf(number)) == 0;
        }
    }

}
